let {classes: Cc, interfaces: Ci, utils: Cu} = Components;

Cu.import("resource:///modules/experiments/Experiments.jsm");
Cu.import("resource://gre/modules/Task.jsm");
Cu.import("resource://gre/modules/Preferences.jsm");

var gStarted = false;

const kSELF_ID = "intersection-observer-beta54@experiments.mozilla.org";
const prefs = new Preferences();

function startup() {
  // Seems startup() function is launched twice after install, we're
  // unsure why so far. We only want it to run once.
  if (gStarted) {
    return;
  }
  gStarted = true;

  Task.spawn(function*() {
    let branch = yield Experiments.instance().getExperimentBranch(kSELF_ID);
    if (branch == null) {
      let r = Math.random() * 2;
      if (r < 1) {
        branch = "control";
      } else {
        branch = "enabled";
        prefs.set("dom.IntersectionObserver.enabled", true);
      }
      yield Experiments.instance().setExperimentBranch(kSELF_ID, branch);
    } else {
      // Exclude the user if they've manually changed the pref
      let pref = prefs.get("dom.IntersectionObserver.enabled");
      let isBranchValid = true;
      if (branch == "enabled") {
        isBranchValid = (pref == true);
      } else if (branch == "control") {
        isBranchValid = (pref == false);
      }
      if (!isBranchValid) {
        branch = "usermod";
        yield Experiments.instance().setExperimentBranch(kSELF_ID, branch);
      }
    }
  }).then(
    function() {
    },
    function(e) {
      Cu.reportError("Got error during bootstrap startup: " + e);
    });
}

function shutdown(reason) {
  if (reason == ADDON_DISABLE || reason == ADDON_UNINSTALL) {
    // when the add-on is being disabled/uninstalled
    prefs.reset("dom.IntersectionObserver.enabled");
  }
}

function install() {
  // no-op
}

function uninstall() {
  prefs.reset("dom.IntersectionObserver.enabled");
}
